@@ -54,7 +54,7 @@ def create_processing(request_id, workload_id, transform_id, status=ProcessingSt
 
 @transactional_session
 def add_processing(request_id, workload_id, transform_id, status=ProcessingStatus.New,
-                   locking=ProcessingLocking.Idle, submitter=None,
+                   locking=ProcessingLocking.Idle, submitter=None, substatus=ProcessingStatus.New,
                    granularity=None, granularity_type=GranularityType.File, expired_at=None, processing_metadata=None,
                    output_metadata=None, session=None):
     """
@@ -78,7 +78,7 @@ def add_processing(request_id, workload_id, transform_id, status=ProcessingStatu
     """
     try:
         new_processing = create_processing(request_id=request_id, workload_id=workload_id, transform_id=transform_id,
-                                           status=status, locking=locking, submitter=submitter,
+                                           status=status, substatus=substatus, locking=locking, submitter=submitter,
                                            granularity=granularity, granularity_type=granularity_type, expired_at=expired_at,
                                            processing_metadata=processing_metadata, output_metadata=output_metadata)
         new_processing.save(session=session)
@@ -235,12 +235,12 @@ def get_processings_by_status(status, period=None, processing_ids=[], locking=Fa
             query = query.filter(models.Processing.substatus.in_(status))
         else:
             query = query.filter(models.Processing.status.in_(status))
-        query = query.filter(models.Processing.next_poll_at < datetime.datetime.utcnow())
+            query = query.filter(models.Processing.next_poll_at <= datetime.datetime.utcnow())
 
         if processing_ids:
             query = query.filter(models.Processing.processing_id.in_(processing_ids))
-        if period:
-            query = query.filter(models.Processing.updated_at < datetime.datetime.utcnow() - datetime.timedelta(seconds=period))
+        # if period:
+        #     query = query.filter(models.Processing.updated_at < datetime.datetime.utcnow() - datetime.timedelta(seconds=period))
         if locking:
             query = query.filter(models.Processing.locking == ProcessingLocking.Idle)
         if submitter:
